;**********************************************************************
;   								      *
;    Gestion d'un seul PWM                                            *
;    Precision de 256 increments                                      *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    tourelle.asm                                      *
;    Date:          24/8/99                                           *
;    File Version:  1                                                 *
;                                                                     *
;    Author:        Julien                                            *
;    Company:       Fribotte                                          *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required: rien                                             *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes: Quartz 4 Mhz                                              *
;                                                                     *
;                                                                     *
;**********************************************************************


	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC
; Pas de code protege, pas de watch hdog timer, delai de demarrage, horloge XT.


;**********************************************************************
;                                                                     *
;    Variables                                                        *
;                                                                     *
;                                                                     *
;**********************************************************************

;***** VARIABLE interruption
w_temp        	EQU     0x0C        	; variable used for context saving 
status_temp   	EQU     0x0D        	; variable used for context saving


;***** VARIABLE PWM
Pwm		EQU	0x10		; Valeur du PWM a appliquer entre 0 et 256.

;***** VARIABLE TIMER
Phase		EQU	0x13		; Phase 0 ou 1 en sortie du PWM

;***** VARIABLE TIMER
tmp_cmp		EQU	0x14		; Compteur temporaire pour la seconde


;**************************************************************** DEBUT
		ORG     0x000           ; processor reset vector
  		GOTO    main            ; go to beginning of program



;**********************************************************************
;                                                                     *
;    Interruption TimeR                                               *
;                                                                     *
;                                                                     *
;**********************************************************************

		ORG     0x004           ; interrupt vector location

;***** Sauvegarde contexte
		MOVWF   w_temp          ; save off current W register contents
		MOVF	STATUS,0        ; move status register into W register
		MOVWF	status_temp     ; save off contents of STATUS register

;***** On incrmemente le tmp_cmp pour tester
		INCF	tmp_cmp,1	; on incremente tmp_cmp
		BTFSC	tmp_cmp,6	; on teste le bit 6 de tmp_cmp (64)
		GOTO	PwmInc		; a 1 on incremente
		GOTO	finTestcmp	; a 0 on continu
PwmInc		INCF 	Pwm,1		; On incremente le PWm
		MOVLW	0x0		; On remet le tmp_cmp a 0
		MOVWF	tmp_cmp		;	/
finTestcmp	NOP

;***** Test de la phase actuelle
		BTFSC	Phase,0		; Phase a 0 ou a 1 ?
		GOTO	Phase0		;	A 1, on passe a la phase 0
		GOTO	Phase1		;	A 0, on passe a la phase 1

;***** Quand la phase doit repasser a 0
Phase0
		MOVLW	0x00		; On repasse B a 0
		MOVWF	PORTB		;	/
	
;***** On regle le prescaler a 128
		BSF	STATUS, RP0	; Bank 1 selectonnee
		MOVLW	B'10000110'	; Timer interne, prescaler sur Timer, Prescaler a 128.
		MOVWF	OPTION_REG	;	/
		BCF	STATUS, RP0	; Bank 0 selectonnee

		MOVLW	0x8C		; Nouveau reglage du timer a 116 (128*(256-116) = 18 ms en tout)
		MOVWF	TMR0		; 	/

		MOVLW	0x00		; On repasse la Phase a 0
		MOVWF	Phase		;	/

		GOTO	FinInter

;***** Quand la phase doit repasser a 1
Phase1
		MOVLW	0xFF		; On repasse B a 255
		MOVWF	PORTB		;	/

;***** On regle le prescaler a 8
		BSF	STATUS, RP0	; Bank 1 selectonnee
		MOVLW	B'10000010'	; Timer interne, prescaler sur Timer, Prescaler a 8.
		MOVWF	OPTION_REG	;	/
		BCF	STATUS, RP0	; Bank 0 selectonnee

		MOVF	Pwm,0		; Nouveau reglage du timer
		SUBLW	0xFF		; On enleve 255 au PWM
		MOVWF	TMR0		; On le regle a la valeur PWM

		MOVLW	0x01		; On repasse la Phase a 1
		MOVWF	Phase		;	/


;***** Redemarage interruption
FinInter	BCF 	INTCON,	T0IF	; On efface l'interruption TimeR pour la relance


;***** Restitution contexte
		MOVF    status_temp,0   ; retrieve copy of STATUS register
		MOVWF	STATUS          ; restore pre-isr STATUS register contents
		SWAPF   w_temp,f
		SWAPF   w_temp,w        ; restore pre-isr W register contents
		RETFIE                  ; return from interrupt



;**********************************************************************
;                                                                     *
;    Programme Main                                                   *
;                                                                     *
;                                                                     *
;**********************************************************************


main

	CALL	InitIO				; On intialise les ports d'E/S (A et B)
	CALL 	InitRtcc			; On intialise l'interruption TimeR
	MOVLW	0x64				; On met le PWM a 100 (correspond a 0.8ms)
	MOVWF	Pwm				;	/
	
boucle
	GOTO 	boucle				; Boucle infinie du main



;**********************************************************************
;                                                                     *
;    Initialisation des ports d'E/S                           	      *
;                                                                     *
;                                                                     *
;**********************************************************************


InitIO
	CLRF	PORTB		; Initialise le port A
	BSF	STATUS, RP0	; Bank 1 selectonnee
	MOVLW	0x0		; Port RB7-RB0 en sortie
	MOVWF	TRISB		;	/
	BCF 	STATUS,	RP0	; Bank 0 selectonnee

	CLRF 	PORTA		; Initialise le port A
	BSF 	STATUS,	RP0	; Bank 1 selectonnee
	MOVLW	0x1F		; Port RA4-RA0 en entree RA5 en sortie
	MOVWF	TRISA		;	/
	BCF 	STATUS,	RP0	; Bank 0 selectonnee
	RETURN



;**********************************************************************
;                                                                     *
;    Initialisation de l'interruption TimeR                           *
;                                                                     *
;                                                                     *
;**********************************************************************


InitRtcc

;*****  On regle le prescaler
	CLRWDT			; Clear WDT and prescaler
	BSF	STATUS, RP0	; Bank 1 selectonnee
	MOVLW	B'10000010'	; Timer interne, prescaler sur Timer, Prescaler a 8.
	MOVWF	OPTION_REG	;	/
	BCF	STATUS, RP0	; Bank 0 selectonnee

;*****  On regle l'interruption

	MOVLW	0xFA		; Initialisation de la valeur du TimeR
	MOVWF	TMR0		;	/

	MOVLW   B'10100000'     ; enable interrupts
	MOVWF   INTCON          ;       /
	RETURN



;**********************************************************************
;                                                                     *
;    Fin				                              *
;                                                                     *
;                                                                     *
;**********************************************************************


	END                     ; directive 'end of program'


